==> Z-Wave_SDK_04.54.02.zip: data/DevKit/Product/dev_ctrl/p_button.c <==
/******************************* p_button.c  *******************************
 *           #######
 *           ##  ##
 *           #  ##    ####   #####    #####  ##  ##   #####
 *             ##    ##  ##  ##  ##  ##      ##  ##  ##
 *            ##  #  ######  ##  ##   ####   ##  ##   ####
 *           ##  ##  ##      ##  ##      ##   #####      ##
 *          #######   ####   ##  ##  #####       ##  #####
 *                                           #####
 *          Z-Wave, the wireless language.
 *
 *              Copyright (c) 2001
 *              Zensys A/S
 *              Denmark
 *
 *              All Rights Reserved
 *
 *    This source file is subject to the terms and conditions of the
 *    Zensys Software License Agreement which restricts the manner
 *    in which it may be used.
 *
 *---------------------------------------------------------------------------
 *
 * Description: Button module for development kit controller board.
 *
 * Author:   Henrik Holm
 *
 * Last Changed By:  $Author: heh $
 * Revision:         $Revision: 4963 $
 * Last Changed:     $Date: 2004-04-07 13:00:28 +0200 (on, 07 apr 2004) $
 *
 ****************************************************************************/

/****************************************************************************/
/*                              INCLUDE FILES                               */
/****************************************************************************/
#include <p_button.h>
#include <ZW_controller_api.h>
#include <ZW_pindefs.h>
#include <ZW_uart_api.h>

/****************************************************************************/
/*                      PRIVATE TYPES and DEFINITIONS                       */
/****************************************************************************/

/*Timing used when checking button status*/
#define PB_DEBOUNCE_TIME 2      /*10-20ms debounce should be enough*/
#define PB_LONG_TIME 60         /*600ms before we assume the key is held down*/

#define PB_REPEAT_WAIT_TIME   20 /* Wait 200ms before repeating */

/*If the button is pressed set bit to 1*/
#define SET_PB_STATUS(val) if(PIN_GET(val##)==0) bPressed |= DOWN_##val

#define IS_DOWN_REPEAT(val) ((bPressed&DOWN_##val)&&(bPressed&PB_REPEAT))
#define IS_DOWN_HELD(val) ((bPressed&DOWN_##val)&&(bPressed&PB_HELD))

/****************************************************************************/
/*                              PRIVATE DATA                                */
/****************************************************************************/
BYTE bPressed = FALSE;
BYTE bOldPressed = FALSE;
BYTE readyCount = 0;

/****************************************************************************/
/*                           EXPORTED FUNCTIONS                             */
/****************************************************************************/
/*============================   TimerCheckButtons   ======================
**    Function description
**    This function checks and updates the status of the Push buttons.
**    It is run every 10ms
**    Side effects:
**
**--------------------------------------------------------------------------*/
void TimerCheckButtons(void)
{
  if(PIN_GET(EEPCS)!=0) /*Only check buttons if EEPROM is not selected*/
  {
    if(bPressed)
      bOldPressed = bPressed&PB_MASK; //Store old values if a key is down

    bPressed = FALSE;
  /*Call macro to set value of PBs*/
    PIN_IN(PB0,1);
    PIN_IN(PB1,1);
    PIN_IN(PB2,1);
    PIN_IN(PB3,1);
#ifndef ZW_DEBUG
#ifndef ZW_DEBUG_CMD
#ifndef ZW_DEBUG_PROTOCOL
    PIN_IN(PB4,1);
#endif /*ZW_DEBUG_PROTOCOL*/
#endif
#endif /*ZW_DEBUG*/
    SET_PB_STATUS(PB0);
    SET_PB_STATUS(PB1);
    SET_PB_STATUS(PB2);
    SET_PB_STATUS(PB3);
#ifndef ZW_DEBUG
#ifndef ZW_DEBUG_CMD
#ifndef ZW_DEBUG_PROTOCOL
    SET_PB_STATUS(PB4);
#endif /*ZW_DEBUG_PROTOCOL*/
#endif
#endif /*ZW_DEBUG*/
  }
  /*Make sure that MOSI is output after we checked the key. The rest is ok as inputs*/
  PIN_OUT(MOSI);

  if(bPressed)
    readyCount++; /*Increment readyCount if PB is down*/
  else
  {
    /*A key has been released, increment debounce*/
    if(bOldPressed&PB_RELEASE)
      readyCount++;
    else if((bOldPressed&PB_MASK))
    {
      /*If last detection resulted in a value. Restart DEBOUNCE*/
      readyCount = 0;
      bPressed = 0;    /*Zero bPressed*/
      bOldPressed |= PB_RELEASE;         /*Set bit, release detect ongoing*/
    }

  }
  if(readyCount == 0xFF)
    readyCount = 0xFE;

  if(readyCount>PB_DEBOUNCE_TIME)
    bOldPressed = FALSE;              /*Zero old values. No need when PB_DEBOUNCE expires*/
  if(readyCount>PB_REPEAT_WAIT_TIME)
    bPressed |= PB_REPEAT;
  if(readyCount>PB_LONG_TIME)
    bPressed |= PB_HELD;
}
/*============================   ButtonPressed   ============================
**    Function description
**     Returns the status of the Push buttons
**    Side effects:
**
**--------------------------------------------------------------------------*/
BYTE ButtonPressed(void)
{
 if(readyCount>PB_DEBOUNCE_TIME)
   return bPressed;
 else
   return bOldPressed;
}

==> Z-Wave_SDK_06.02.00.zip: data/DevKit/Product/dev_ctrl/p_button.c <==
/******************************* p_button.c  *******************************
 *           #######
 *           ##  ##
 *           #  ##    ####   #####    #####  ##  ##   #####
 *             ##    ##  ##  ##  ##  ##      ##  ##  ##
 *            ##  #  ######  ##  ##   ####   ##  ##   ####
 *           ##  ##  ##      ##  ##      ##   #####      ##
 *          #######   ####   ##  ##  #####       ##  #####
 *                                           #####
 *          Z-Wave, the wireless language.
 *
 *              Copyright (c) 2001
 *              Zensys A/S
 *              Denmark
 *
 *              All Rights Reserved
 *
 *    This source file is subject to the terms and conditions of the
 *    Zensys Software License Agreement which restricts the manner
 *    in which it may be used.
 *
 *---------------------------------------------------------------------------
 *
 * Description: Button module for development kit controller board.
 *
 * Author:   Henrik Holm
 *
 * Last Changed By:  $Author: psh $
 * Revision:         $Revision: 22659 $
 * Last Changed:     $Date: 2012-05-02 17:09:49 +0200 (Wed, 02 May 2012) $
 *
 ****************************************************************************/
#ifdef PATCH_ENABLE
/****************************************************************************/
/* Include assembly MACRO definitions for patch insertions.                 */
/*                                                                          */
/* Define $SET (MAKE_PATCHABLE_CODE) for making patchable code destinned    */
/* for OTP or ROM memory.                                                   */
/* Undefine $RESET (MAKE_PATCHABLE_CODE) for making code containing patch   */
/* code destinned for RAM or FLASH memory.                                  */
/****************************************************************************/
#if defined(WORK_PATCH) || defined(STARTER_PATCH)
/* Making code containing patch code destinned for development RAM memory.  */
#pragma asm
$RESET (MAKE_PATCHABLE_CODE)
$INCLUDE (ZW_patch.inc)
#pragma endasm
/* Rename CODE class to CODE_PATCH */
#pragma userclass (code = PATCH)
/* Rename CONST class to CONST_PATCH */
#pragma userclass (const = PATCH)
/* Rename XDATA class to XDATA_PATCH */
#pragma userclass (xdata = PATCH)
#else
/* Making patchable code destinned for OTP or ROM memory.                   */
#pragma asm
$SET (MAKE_PATCHABLE_CODE)
$INCLUDE (ZW_patch.inc)
#pragma endasm
#endif /* elsif defined(WORK_PATCH) || defined(STARTER_PATCH) */
#endif /* PATCH_ENABLE */

/****************************************************************************/
/*                              INCLUDE FILES                               */
/****************************************************************************/
#include <ZW_patch.h>
#include <p_button.h>
#include <ZW_controller_api.h>
#include <ZW_pindefs.h>
#include <ZW_uart_api.h>

/****************************************************************************/
/*                      PRIVATE TYPES and DEFINITIONS                       */
/****************************************************************************/

/*Timing used when checking button status*/
#define PB_DEBOUNCE_TIME 2      /*10-20ms debounce should be enough*/
#define PB_LONG_TIME 60         /*600ms before we assume the key is held down*/

#define PB_REPEAT_WAIT_TIME   20 /* Wait 200ms before repeating */

/*If the button is pressed set bit to 1*/
#define SET_PB_STATUS(val) if(PIN_GET(val##)==0) bPressed |= DOWN_##val

#define IS_DOWN_REPEAT(val) ((bPressed&DOWN_##val)&&(bPressed&PB_REPEAT))
#define IS_DOWN_HELD(val) ((bPressed&DOWN_##val)&&(bPressed&PB_HELD))

/****************************************************************************/
/*                              PRIVATE DATA                                */
/****************************************************************************/
PATCH_VARIABLE BYTE bPressed
#ifndef WORK_PATCH
 = FALSE
#endif
;
PATCH_VARIABLE BYTE bOldPressed
#ifndef WORK_PATCH
 = FALSE
#endif
;
PATCH_VARIABLE BYTE readyCount
#ifndef WORK_PATCH
 = 0
#endif
;
/****************************************************************************/
/*                           EXPORTED FUNCTIONS                             */
/****************************************************************************/
/*============================   TimerCheckButtons   ======================
**    Function description
**    This function checks and updates the status of the Push buttons.
**    It is run every 10ms
**    Side effects:
**
**--------------------------------------------------------------------------*/
void
PATCH_FUNCTION_NAME(TimerCheckButtons)(void)
#ifdef PATCH_ENABLE
reentrant
#endif
{
#ifdef PATCH_ENABLE
#pragma asm
PATCH_TABLE_ENTRY(TimerCheckButtons)
#pragma endasm
#endif
  if(PIN_GET(EEPCS)!=0) /*Only check buttons if EEPROM is not selected*/
  {
    if(bPressed)
      bOldPressed = bPressed&PB_MASK; //Store old values if a key is down

    bPressed = FALSE;
  /*Call macro to set value of PBs*/
    PIN_IN(PB0,1);
    PIN_IN(PB1,1);
    PIN_IN(PB2,1);
    PIN_IN(PB3,1);
#ifdef ZW040x
    PIN_IN(PB4,1);
#else
#ifndef ZW_DEBUG
#ifndef ZW_DEBUG_CMD
#ifndef ZW_DEBUG_PROTOCOL
    PIN_IN(PB4,1);
#endif /*ZW_DEBUG_PROTOCOL*/
#endif
#endif /*ZW_DEBUG*/
#endif /*ZW040x*/

    SET_PB_STATUS(PB0);
    SET_PB_STATUS(PB1);
    SET_PB_STATUS(PB2);
    SET_PB_STATUS(PB3);
#ifdef ZW040x
    SET_PB_STATUS(PB4);
#else
#ifndef ZW_DEBUG
#ifndef ZW_DEBUG_CMD
#ifndef ZW_DEBUG_PROTOCOL
    SET_PB_STATUS(PB4);
#endif /*ZW_DEBUG_PROTOCOL*/
#endif
#endif /*ZW_DEBUG*/
#endif /*ZW040x*/
  }

#ifdef ZW040x
  /*Make sure that MOSI is output after we checked the key. The rest is ok as inputs*/
  PIN_OUT(SPI1_SCK); // PB4 (P24) is used as SCK output to the EEPROM
  PIN_OUT(SPI1_MOSI); // PB4 (P22) is used as MOSI output to the EEPROM
#else
  /*Make sure that MOSI is output after we checked the key. The rest is ok as inputs*/
  PIN_OUT(MOSI);
#endif

  if(bPressed)
    readyCount++; /*Increment readyCount if PB is down*/
  else
  {
    /*A key has been released, increment debounce*/
    if(bOldPressed&PB_RELEASE)
      readyCount++;
    else if((bOldPressed&PB_MASK))
    {
      /*If last detection resulted in a value. Restart DEBOUNCE*/
      readyCount = 0;
      bPressed = 0;    /*Zero bPressed*/
      bOldPressed |= PB_RELEASE;         /*Set bit, release detect ongoing*/
    }

  }
  if(readyCount == 0xFF)
    readyCount = 0xFE;

  if(readyCount>PB_DEBOUNCE_TIME)
    bOldPressed = FALSE;              /*Zero old values. No need when PB_DEBOUNCE expires*/
  if(readyCount>PB_REPEAT_WAIT_TIME)
    bPressed |= PB_REPEAT;
  if(readyCount>PB_LONG_TIME)
    bPressed |= PB_HELD;
}
/*============================   ButtonPressed   ============================
**    Function description
**     Returns the status of the Push buttons
**    Side effects:
**
**--------------------------------------------------------------------------*/
BYTE
PATCH_FUNCTION_NAME(ButtonPressed)(void)
#ifdef PATCH_ENABLE
reentrant
#endif
{
#ifdef PATCH_ENABLE_
#pragma asm
PATCH_TABLE_ENTRY(ButtonPressed)
#pragma endasm
#endif
 if(readyCount>PB_DEBOUNCE_TIME)
   return bPressed;
 else
   return bOldPressed;
}
